@@ -44,13 +44,14 @@
 //  W A R N I N G
 //  -------------
 //
-// This file is not part of the Qt API.  It exists for the convenience
-// of the QLibrary class.  This header file may change from
-// version to version without notice, or even be removed.
+// This file is not part of the Qt API. It exists purely as an
+// implementation detail. This header file may change from version to
+// version without notice, or even be removed.
 //
 // We mean it.
 //
 
+#include <QtGui/private/qtguiglobal_p.h>
 #include <QtCore/QStringList>
 #include <QtCore/QVector>
 #include <QtCore/QVariant>
@@ -74,6 +75,10 @@ QT_END_NAMESPACE
 #if defined(Q_OS_INTEGRITY)
 #  undef Value
 #endif
+// Hurd has #define TILDE 0x00080000 from <sys/ioctl.h>
+#if defined(TILDE)
+#  undef TILDE
+#endif
 
 #define QT_CSS_DECLARE_TYPEINFO(Class, Type) \
     } /* namespace QCss */ \
@@ -117,6 +122,7 @@ enum Property {
     BorderRight,
     BorderTop,
     BorderBottom,
+    BorderCollapse,
     Padding,
     PaddingLeft,
     PaddingRight,
@@ -190,6 +196,9 @@ enum Property {
     QtListNumberSuffix,
     LineHeight,
     QtLineHeightType,
+    FontKerning,
+    QtForegroundTextureCacheKey,
+    QtIcon,
     NumProperties
 };
 
@@ -198,6 +207,7 @@ enum KnownValue {
     Value_Normal,
     Value_Pre,
     Value_NoWrap,
+    Value_PreLine,
     Value_PreWrap,
     Value_Small,
     Value_Medium,
@@ -461,8 +471,8 @@ struct Q_GUI_EXPORT Declaration
     Attachment attachmentValue() const;
     int  styleFeaturesValue() const;
 
-    bool intValue(int *i, const char *unit = 0) const;
-    bool realValue(qreal *r, const char *unit = 0) const;
+    bool intValue(int *i, const char *unit = nullptr) const;
+    bool realValue(qreal *r, const char *unit = nullptr) const;
 
     QSize sizeValue() const;
     QRect rectValue() const;
@@ -470,6 +480,7 @@ struct Q_GUI_EXPORT Declaration
     QIcon iconValue() const;
 
     void borderImageValue(QString *image, int *cuts, TileMode *h, TileMode *v) const;
+    bool borderCollapseValue() const;
 };
 QT_CSS_DECLARE_TYPEINFO(Declaration, Q_MOVABLE_TYPE)
 
@@ -538,8 +549,11 @@ struct AttributeSelector
     enum ValueMatchType {
         NoMatch,
         MatchEqual,
-        MatchContains,
-        MatchBeginsWith
+        MatchIncludes,
+        MatchDashMatch,
+        MatchBeginsWith,
+        MatchEndsWith,
+        MatchContains
     };
     inline AttributeSelector() : valueMatchCriterium(NoMatch) {}
 
@@ -557,7 +571,8 @@ struct BasicSelector
         NoRelation,
         MatchNextSelectorIfAncestor,
         MatchNextSelectorIfParent,
-        MatchNextSelectorIfPreceeds
+        MatchNextSelectorIfDirectAdjecent,
+        MatchNextSelectorIfIndirectAdjecent,
     };
 
     QString elementName;
@@ -574,7 +589,7 @@ struct Q_GUI_EXPORT Selector
 {
     QVector<BasicSelector> basicSelectors;
     int specificity() const;
-    quint64 pseudoClass(quint64 *negated = 0) const;
+    quint64 pseudoClass(quint64 *negated = nullptr) const;
     QString pseudoElement() const;
 };
 QT_CSS_DECLARE_TYPEINFO(Selector, Q_MOVABLE_TYPE)
@@ -646,7 +661,7 @@ public:
     };
 
     QVector<StyleRule> styleRulesForNode(NodePtr node);
-    QVector<Declaration> declarationsForNode(NodePtr node, const char *extraPseudo = 0);
+    QVector<Declaration> declarationsForNode(NodePtr node, const char *extraPseudo = nullptr);
 
     virtual bool nodeNameEquals(NodePtr node, const QString& nodeName) const;
     virtual QString attribute(NodePtr node, const QString &name) const = 0;
@@ -664,7 +679,7 @@ public:
     Qt::CaseSensitivity nameCaseSensitivity;
 private:
     void matchRule(NodePtr node, const StyleRule &rules, StyleSheetOrigin origin,
-                    int depth, QMap<uint, StyleRule> *weightedRules);
+                    int depth, QMultiMap<uint, StyleRule> *weightedRules);
     bool selectorMatches(const Selector &rule, NodePtr node);
     bool basicSelectorMatches(const BasicSelector &rule, NodePtr node);
 };
@@ -678,11 +693,15 @@ enum TokenType {
     CDC,
     INCLUDES,
     DASHMATCH,
+    BEGINSWITH,
+    ENDSWITH,
+    CONTAINS,
 
     LBRACE,
     PLUS,
     GREATER,
     COMMA,
+    TILDE,
 
     STRING,
     INVALID,
@@ -730,7 +749,7 @@ QT_CSS_DECLARE_TYPEINFO(Symbol, Q_MOVABLE_TYPE)
 class Q_GUI_EXPORT Scanner
 {
 public:
-    static QString preprocess(const QString &input, bool *hasEscapeSequences = 0);
+    static QString preprocess(const QString &input, bool *hasEscapeSequences = nullptr);
     static void scan(const QString &preprocessedInput, QVector<Symbol> *symbols);
 };
 
@@ -782,7 +801,7 @@ public:
     inline bool testImport() { return testTokenAndEndsWith(ATKEYWORD_SYM, QLatin1String("import")); }
     inline bool testMedia() { return testTokenAndEndsWith(ATKEYWORD_SYM, QLatin1String("media")); }
     inline bool testPage() { return testTokenAndEndsWith(ATKEYWORD_SYM, QLatin1String("page")); }
-    inline bool testCombinator() { return test(PLUS) || test(GREATER) || test(S); }
+    inline bool testCombinator() { return test(PLUS) || test(GREATER) || test(TILDE) || test(S); }
     inline bool testProperty() { return test(IDENT); }
     bool testTerm();
     inline bool testExpr() { return testTerm(); }
@@ -831,20 +850,21 @@ struct Q_GUI_EXPORT ValueExtractor
     bool extractGeometry(int *w, int *h, int *minw, int *minh, int *maxw, int *maxh);
     bool extractPosition(int *l, int *t, int *r, int *b, QCss::Origin *, Qt::Alignment *,
                          QCss::PositionMode *, Qt::Alignment *);
-    bool extractBox(int *margins, int *paddings, int *spacing = 0);
+    bool extractBox(int *margins, int *paddings, int *spacing = nullptr);
     bool extractBorder(int *borders, QBrush *colors, BorderStyle *Styles, QSize *radii);
     bool extractOutline(int *borders, QBrush *colors, BorderStyle *Styles, QSize *radii, int *offsets);
     bool extractPalette(QBrush *fg, QBrush *sfg, QBrush *sbg, QBrush *abg);
     int  extractStyleFeatures();
     bool extractImage(QIcon *icon, Qt::Alignment *a, QSize *size);
+    bool extractIcon(QIcon *icon, QSize *size);
 
-    int lengthValue(const Declaration &decl);
+    void lengthValues(const Declaration &decl, int *m);
 
 private:
     void extractFont();
     void borderValue(const Declaration &decl, int *width, QCss::BorderStyle *style, QBrush *color);
     LengthData lengthValue(const Value& v);
-    void lengthValues(const Declaration &decl, int *m);
+    int lengthValue(const Declaration &decl);
     QSize sizeValue(const Declaration &decl);
     void sizeValues(const Declaration &decl, QSize *radii);
 
