@@ -55,17 +55,21 @@
 #include "qnetworkaccessmanager.h"
 #include "qnetworkaccesscache_p.h"
 #include "qnetworkaccessbackend_p.h"
+#include "private/qnetconmonitor_p.h"
 #include "qnetworkrequest.h"
-#include "qhstsstore_p.h"
 #include "qhsts_p.h"
 #include "private/qobject_p.h"
 #include "QtNetwork/qnetworkproxy.h"
 #include "QtNetwork/qnetworksession.h"
 #include "qnetworkaccessauthenticationmanager_p.h"
-#ifndef QT_NO_BEARERMANAGEMENT
+#ifndef QT_NO_BEARERMANAGEMENT // ### Qt6: Remove section
 #include "QtNetwork/qnetworkconfigmanager.h"
 #endif
 
+#if QT_CONFIG(settings)
+#include "qhstsstore_p.h"
+#endif // QT_CONFIG(settings)
+
 QT_BEGIN_NAMESPACE
 
 class QAuthenticator;
@@ -77,12 +81,13 @@ class QNetworkAccessManagerPrivate: public QObjectPrivate
 {
 public:
     QNetworkAccessManagerPrivate()
-        : networkCache(0), cookieJar(0),
-          thread(0),
+        : networkCache(nullptr),
+          cookieJar(nullptr),
+          thread(nullptr),
 #ifndef QT_NO_NETWORKPROXY
-          proxyFactory(0),
+          proxyFactory(nullptr),
 #endif
-#ifndef QT_NO_BEARERMANAGEMENT
+#ifndef QT_NO_BEARERMANAGEMENT // ### Qt6: Remove section
           lastSessionState(QNetworkSession::Invalid),
           networkConfiguration(networkConfigurationManager.defaultConfiguration()),
           customNetworkConfiguration(false),
@@ -97,7 +102,7 @@ public:
           redirectPolicy(QNetworkRequest::ManualRedirectPolicy),
           authenticationManager(QSharedPointer<QNetworkAccessAuthenticationManager>::create())
     {
-#ifndef QT_NO_BEARERMANAGEMENT
+#ifndef QT_NO_BEARERMANAGEMENT // ### Qt6: Remove section
         // we would need all active configurations to check for
         // d->networkConfigurationManager.isOnline(), which is asynchronous
         // and potentially expensive. We can just check the configuration here
@@ -115,8 +120,8 @@ public:
     QThread * createThread();
     void destroyThread();
 
-    void _q_replyFinished();
-    void _q_replyEncrypted();
+    void _q_replyFinished(QNetworkReply *reply);
+    void _q_replyEncrypted(QNetworkReply *reply);
     void _q_replySslErrors(const QList<QSslError> &errors);
     void _q_replyPreSharedKeyAuthenticationRequired(QSslPreSharedKeyAuthenticator *authenticator);
     QNetworkReply *postProcess(QNetworkReply *reply);
@@ -130,7 +135,7 @@ public:
                                 bool allowAuthenticationReuse = true);
     void cacheCredentials(const QUrl &url, const QAuthenticator *auth);
     QNetworkAuthenticationCredential *fetchCachedCredentials(const QUrl &url,
-                                                             const QAuthenticator *auth = 0);
+                                                             const QAuthenticator *auth = nullptr);
 
 #ifndef QT_NO_NETWORKPROXY
     void proxyAuthenticationRequired(const QUrl &url,
@@ -140,14 +145,15 @@ public:
                                 QNetworkProxy *lastProxyAuthentication);
     void cacheProxyCredentials(const QNetworkProxy &proxy, const QAuthenticator *auth);
     QNetworkAuthenticationCredential *fetchCachedProxyCredentials(const QNetworkProxy &proxy,
-                                                             const QAuthenticator *auth = 0);
+                                                             const QAuthenticator *auth = nullptr);
     QList<QNetworkProxy> queryProxy(const QNetworkProxyQuery &query);
 #endif
 
     QNetworkAccessBackend *findBackend(QNetworkAccessManager::Operation op, const QNetworkRequest &request);
     QStringList backendSupportedSchemes() const;
 
-#ifndef QT_NO_BEARERMANAGEMENT
+    void _q_onlineStateChanged(bool isOnline);
+#ifndef QT_NO_BEARERMANAGEMENT // ### Qt6: Remove section
     void createSession(const QNetworkConfiguration &config);
     QSharedPointer<QNetworkSession> getNetworkSession() const;
 
@@ -156,15 +162,16 @@ public:
     void _q_networkSessionPreferredConfigurationChanged(const QNetworkConfiguration &config,
                                                         bool isSeamless);
     void _q_networkSessionStateChanged(QNetworkSession::State state);
-    void _q_onlineStateChanged(bool isOnline);
+
     void _q_configurationChanged(const QNetworkConfiguration &configuration);
     void _q_networkSessionFailed(QNetworkSession::SessionError error);
 
     QSet<QString> onlineConfigurations;
-
 #endif
 
+#if QT_CONFIG(http)
     QNetworkRequest prepareMultipart(const QNetworkRequest &request, QHttpMultiPart *multiPart);
+#endif
 
     // this is the cache for storing downloaded files
     QAbstractNetworkCache *networkCache;
@@ -179,7 +186,7 @@ public:
     QNetworkProxyFactory *proxyFactory;
 #endif
 
-#ifndef QT_NO_BEARERMANAGEMENT
+#ifndef QT_NO_BEARERMANAGEMENT // ### Qt6: Remove section
     QSharedPointer<QNetworkSession> networkSessionStrongRef;
     QWeakPointer<QNetworkSession> networkSessionWeakRef;
     QNetworkSession::State lastSessionState;
@@ -193,6 +200,8 @@ public:
     int activeReplyCount;
     bool online;
     bool initializeSession;
+#else
+    bool networkAccessible = true;
 #endif
 
     bool cookieJarCreated;
@@ -212,10 +221,17 @@ public:
     Q_AUTOTEST_EXPORT static void clearConnectionCache(QNetworkAccessManager *manager);
 
     QHstsCache stsCache;
+#if QT_CONFIG(settings)
     QScopedPointer<QHstsStore> stsStore;
+#endif // QT_CONFIG(settings)
     bool stsEnabled = false;
+    mutable QNetworkStatusMonitor statusMonitor;
 
-#ifndef QT_NO_BEARERMANAGEMENT
+    bool autoDeleteReplies = false;
+
+    int transferTimeout = 0;
+
+#ifndef QT_NO_BEARERMANAGEMENT // ### Qt6: Remove section
     Q_AUTOTEST_EXPORT static const QWeakPointer<const QNetworkSession> getNetworkSession(const QNetworkAccessManager *manager);
 #endif
     Q_DECLARE_PUBLIC(QNetworkAccessManager)
